@@ -3,15 +3,7 @@
 #ifndef __LXC_CGROUP_UTILS_H
 #define __LXC_CGROUP_UTILS_H
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
-#endif
-
-#ifndef FUSE_USE_VERSION
-#define FUSE_USE_VERSION 26
-#endif
-
-#define _FILE_OFFSET_BITS 64
+#include "config.h"
 
 #include <fcntl.h>
 #include <stdarg.h>
@@ -25,7 +17,6 @@
 #include <sys/vfs.h>
 #include <unistd.h>
 
-#include "../config.h"
 #include "../macro.h"
 #include "../memory_utils.h"
 
@@ -70,11 +61,11 @@ extern bool dir_exists(const char *path);
 extern int safe_mount(const char *src, const char *dest, const char *fstype,
 		      unsigned long flags, const void *data, const char *rootfs);
 
-#ifndef HAVE_STRLCPY
+#if !HAVE_STRLCPY
 extern size_t strlcpy(char *, const char *, size_t);
 #endif
 
-#ifndef HAVE_STRLCAT
+#if !HAVE_STRLCAT
 extern size_t strlcat(char *d, const char *s, size_t n);
 #endif
 
