@@ -24,6 +24,7 @@
 #include <sys/mman.h>
 #include <sys/mount.h>
 #include <sys/param.h>
+#include <sys/personality.h>
 #include <sys/socket.h>
 #include <sys/syscall.h>
 #include <sys/sysinfo.h>
@@ -103,7 +104,8 @@ __lxcfs_fuse_ops int proc_getattr(const char *path, struct stat *sb)
 	    strcmp(path, "/proc/stat")		== 0 ||
 	    strcmp(path, "/proc/diskstats")	== 0 ||
 	    strcmp(path, "/proc/swaps")		== 0 ||
-	    strcmp(path, "/proc/loadavg")	== 0) {
+	    strcmp(path, "/proc/loadavg")	== 0 ||
+	    strcmp(path, "/proc/slabinfo")	== 0) {
 		sb->st_size = get_procfile_size(path);
 		sb->st_mode = S_IFREG | 00444;
 		sb->st_nlink = 1;
@@ -117,15 +119,16 @@ __lxcfs_fuse_ops int proc_readdir(const char *path, void *buf,
 				  fuse_fill_dir_t filler, off_t offset,
 				  struct fuse_file_info *fi)
 {
-	if (DIR_FILLER(filler, buf, ".",		NULL, 0) != 0 ||
-	    DIR_FILLER(filler, buf, "..",		NULL, 0) != 0 ||
-	    DIR_FILLER(filler, buf, "cpuinfo",	NULL, 0) != 0 ||
-	    DIR_FILLER(filler, buf, "meminfo",	NULL, 0) != 0 ||
-	    DIR_FILLER(filler, buf, "stat",		NULL, 0) != 0 ||
-	    DIR_FILLER(filler, buf, "uptime",	NULL, 0) != 0 ||
-	    DIR_FILLER(filler, buf, "diskstats",	NULL, 0) != 0 ||
-	    DIR_FILLER(filler, buf, "swaps",	NULL, 0) != 0 ||
-	    DIR_FILLER(filler, buf, "loadavg",	NULL, 0) != 0)
+	if (dir_filler(filler, buf, ".",		0) != 0 ||
+	    dir_filler(filler, buf, "..",		0) != 0 ||
+	    dir_filler(filler, buf, "cpuinfo",		0) != 0 ||
+	    dir_filler(filler, buf, "meminfo",		0) != 0 ||
+	    dir_filler(filler, buf, "stat",		0) != 0 ||
+	    dir_filler(filler, buf, "uptime",		0) != 0 ||
+	    dir_filler(filler, buf, "diskstats",	0) != 0 ||
+	    dir_filler(filler, buf, "swaps",		0) != 0 ||
+	    dir_filler(filler, buf, "loadavg",		0) != 0 ||
+	    dir_filler(filler, buf, "slabinfo",		0) != 0)
 		return -EINVAL;
 
 	return 0;
@@ -150,6 +153,8 @@ __lxcfs_fuse_ops int proc_open(const char *path, struct fuse_file_info *fi)
 		type = LXC_TYPE_PROC_SWAPS;
 	else if (strcmp(path, "/proc/loadavg") == 0)
 		type = LXC_TYPE_PROC_LOADAVG;
+	else if (strcmp(path, "/proc/slabinfo") == 0)
+		type = LXC_TYPE_PROC_SLABINFO;
 	if (type == -1)
 		return -ENOENT;
 
@@ -200,7 +205,8 @@ static uint64_t get_memlimit(const char *cgroup, bool swap)
 	else
 		ret = cgroup_ops->get_memory_max(cgroup_ops, cgroup, &memlimit_str);
 	if (ret > 0 && memlimit_str[0] && safe_uint64(memlimit_str, &memlimit, 10) < 0)
-		lxcfs_error("Failed to convert memlimit %s", memlimit_str);
+		lxcfs_error("Failed to convert memory%s.max=%s for cgroup %s",
+			    swap ? ".swap" : "", memlimit_str, cgroup);
 
 	return memlimit;
 }
@@ -967,7 +973,7 @@ static int proc_stat_read(char *buf, size_t size, off_t offset,
 		char cpu_char[10]; /* That's a lot of cores */
 		char *c;
 		uint64_t all_used, cg_used, new_idle;
-		int ret;
+		int ret, cpu_to_render;
 
 		if (strlen(line) == 0)
 			continue;
@@ -994,6 +1000,11 @@ static int proc_stat_read(char *buf, size_t size, off_t offset,
 
 		curcpu++;
 
+		if (cgroup_ops->can_use_cpuview(cgroup_ops) && opts && opts->use_cfs)
+			cpu_to_render = curcpu;
+		else
+			cpu_to_render = physcpu;
+
 		ret = sscanf(
 			   line,
 			   "%*s"        /* <skip> */
@@ -1022,7 +1033,7 @@ static int proc_stat_read(char *buf, size_t size, off_t offset,
 			if (!c)
 				continue;
 
-			l = snprintf(cache, cache_size, "cpu%d%s", curcpu, c);
+			l = snprintf(cache, cache_size, "cpu%d%s", cpu_to_render, c);
 			if (l < 0)
 				return log_error(0, "Failed to write cache");
 			if ((size_t)l >= cache_size)
@@ -1047,13 +1058,13 @@ static int proc_stat_read(char *buf, size_t size, off_t offset,
 				new_idle = idle + (all_used - cg_used);
 			} else {
 				lxcfs_debug("cpu%d from %s has unexpected cpu time: %" PRIu64 " in /proc/stat, %" PRIu64 " in cpuacct.usage_all; unable to determine idle time",
-					    curcpu, cg, all_used, cg_used);
+					    cpu_to_render, cg, all_used, cg_used);
 				new_idle = idle;
 			}
 
 			l = snprintf(cache, cache_size,
 				     "cpu%d %" PRIu64 " 0 %" PRIu64 " %" PRIu64 " 0 0 0 0 0 0\n",
-				     curcpu, cg_cpu_usage[physcpu].user,
+				     cpu_to_render, cg_cpu_usage[physcpu].user,
 				     cg_cpu_usage[physcpu].system, new_idle);
 			if (l < 0)
 				return log_error(0, "Failed to write cache");
@@ -1086,7 +1097,7 @@ static int proc_stat_read(char *buf, size_t size, off_t offset,
 	int cpuall_len = snprintf(
 			cpuall,
 			CPUALL_MAX_SIZE,
-			"cpu  "
+			"cpu "
 			" %" PRIu64 /* user_sum */
 			" %" PRIu64 /* nice_sum */
 			" %" PRIu64 /* system_sum */
@@ -1426,6 +1437,115 @@ static int proc_meminfo_read(char *buf, size_t size, off_t offset,
 	return total_len;
 }
 
+static int proc_slabinfo_read(char *buf, size_t size, off_t offset,
+			      struct fuse_file_info *fi)
+{
+	__do_free char *cgroup = NULL, *line = NULL;
+	__do_free void *fopen_cache = NULL;
+	__do_fclose FILE *f = NULL;
+	__do_close int fd = -EBADF;
+	struct fuse_context *fc = fuse_get_context();
+	struct file_info *d = INTTYPE_TO_PTR(fi->fh);
+	size_t linelen = 0, total_len = 0;
+	char *cache = d->buf;
+	size_t cache_size = d->buflen;
+	pid_t initpid;
+
+	if (offset) {
+		size_t left;
+
+		if (offset > d->size)
+			return -EINVAL;
+
+		if (!d->cached)
+			return 0;
+
+		left = d->size - offset;
+		total_len = left > size ? size : left;
+		memcpy(buf, cache + offset, total_len);
+
+		return total_len;
+	}
+
+	initpid = lookup_initpid_in_store(fc->pid);
+	if (initpid <= 1 || is_shared_pidns(initpid))
+		initpid = fc->pid;
+
+	cgroup = get_pid_cgroup(initpid, "memory");
+	if (!cgroup)
+		return read_file_fuse("/proc/slabinfo", buf, size, d);
+
+	prune_init_slice(cgroup);
+
+	fd = cgroup_ops->get_memory_slabinfo_fd(cgroup_ops, cgroup);
+	if (fd < 0)
+		return read_file_fuse("/proc/slabinfo", buf, size, d);
+
+	f = fdopen_cached(fd, "re", &fopen_cache);
+	if (!f)
+		return read_file_fuse("/proc/slabinfo", buf, size, d);
+
+	while (getline(&line, &linelen, f) != -1) {
+		ssize_t l = snprintf(cache, cache_size, "%s", line);
+		if (l < 0)
+			return log_error(0, "Failed to write cache");
+		if ((size_t)l >= cache_size)
+			return log_error(0, "Write to cache was truncated");
+
+		cache += l;
+		cache_size -= l;
+		total_len += l;
+	}
+
+	d->cached = 1;
+	d->size = total_len;
+	if (total_len > size)
+		total_len = size;
+	memcpy(buf, d->buf, total_len);
+
+	return total_len;
+}
+
+static int proc_read_with_personality(int (*do_proc_read)(char *, size_t, off_t,
+			     struct fuse_file_info *), char *buf, size_t size, off_t offset,
+			     struct fuse_file_info *fi)
+{
+	struct fuse_context *fc = fuse_get_context();
+	__u32 host_personality = liblxcfs_personality(), caller_personality;
+	bool change_personality;
+	int ret, read_ret;
+
+	if (get_task_personality(fc->pid, &caller_personality) < 0)
+		return log_error(0, "Failed to get caller process (pid: %d) personality", fc->pid);
+
+	/* do we need to change thread personality? */
+	change_personality = host_personality != caller_personality;
+
+	if (change_personality) {
+		ret = personality(caller_personality);
+		if (ret == -1)
+			return log_error(0, "Call to personality(%d) failed: %s\n",
+				caller_personality, strerror(errno));
+
+		lxcfs_debug("task (tid: %d) personality was changed %d -> %d\n",
+				(int)syscall(SYS_gettid), ret, caller_personality);
+	}
+
+	read_ret = do_proc_read(buf, size, offset, fi);
+
+	if (change_personality) {
+		ret = personality(host_personality);
+		if (ret == -1)
+			return log_error(0, "Call to personality(%d) failed: %s\n",
+				host_personality, strerror(errno));
+
+		lxcfs_debug("task (tid: %d) personality was restored %d -> %d\n",
+				(int)syscall(SYS_gettid), ret, host_personality);
+	}
+
+	return read_ret;
+}
+
 __lxcfs_fuse_ops int proc_read(const char *path, char *buf, size_t size,
 			       off_t offset, struct fuse_file_info *fi)
 {
@@ -1440,7 +1560,7 @@ __lxcfs_fuse_ops int proc_read(const char *path, char *buf, size_t size,
 						  buf, size, offset, f);
 	case LXC_TYPE_PROC_CPUINFO:
 		if (liblxcfs_functional())
-			return proc_cpuinfo_read(buf, size, offset, fi);
+			return proc_read_with_personality(&proc_cpuinfo_read, buf, size, offset, fi);
 
 		return read_file_fuse_with_offset(LXC_TYPE_PROC_CPUINFO_PATH,
 						  buf, size, offset, f);
@@ -1474,6 +1594,12 @@ __lxcfs_fuse_ops int proc_read(const char *path, char *buf, size_t size,
 
 		return read_file_fuse_with_offset(LXC_TYPE_PROC_LOADAVG_PATH,
 						  buf, size, offset, f);
+	case LXC_TYPE_PROC_SLABINFO:
+		if (liblxcfs_functional())
+			return proc_slabinfo_read(buf, size, offset, fi);
+
+		return read_file_fuse_with_offset(LXC_TYPE_PROC_SLABINFO_PATH,
+						  buf, size, offset, f);
 	}
 
 	return -EINVAL;
